__author__ = 'heddevanderheide'

# Django specific
from django.conf import settings
from django.db import models

# App specific
from networth.managers import NetworthManager
from networth.mixins import NetworthMixin
from networth.signals import ceiling_increased


class NetworthModel(NetworthMixin, models.Model):
    _networth = models.IntegerField(default=getattr(settings, 'NETWORTH_DEFAULT', 1))
    _relative_networth = models.IntegerField(default=0)

    objects = NetworthManager()

    class Meta:
        abstract = True

    def get_default_networth(self):
        return getattr(settings, 'NETWORTH_DEFAULT', 1)

    def networth(self, realtime=False, commit=False):
        if realtime:
            return self.__networth(commit=commit)
        return self._networth

    def relative_networth(self, realtime=False, commit=False):
        if realtime:
            return self.__relative_networth(commit=commit)
        return self._relative_networth

    def __networth(self, commit=False):
        return self._NetworthMixin__networth(commit=commit)

    def __relative_networth(self, commit=False):
        # returns relative networth (percentual) compared to the
        # highest valued object

        ceiling = self.__class__.objects.exclude(pk=self.pk).ceiling()

        if ceiling == 1:
            n = 100
        else:
            if self._networth == ceiling:
                n = 100
            else:
                n = int((float(self._networth)) / ceiling * 100)

                if n > 100:
                    ceiling_increased.connect(self)

                    n = 100

        if commit:
            self._commit('_relative_networth', n)

        return n

    def _commit(self, field, value):
        setattr(self, field, value)
        self.save()


__author__ = 'heddevanderheide'


class NetworthMixin(object):
    def get_default_networth(self):
        return 1

    def networth(self, realtime=True, commit=False):
        return self.__networth(commit=commit)

    def __networth(self, commit=False):
        n = self.get_default_networth()

        for field in self.__class__.Networth.fields:
            f, values = field

            v = getattr(self, f)

            condition, award = values
            condition_is_callable, award_is_callable = callable(condition), callable(award)

            if condition_is_callable:

                if values[1] == 'result':
                    n += condition(v)

                elif award_is_callable:
                    n += award(condition(v))

                else:
                    if v:
                        n += award

            else:

                if type(condition) == bool:
                    if award_is_callable:
                        n += award(1 if bool(v) == condition else 0)
                    else:
                        n += award

                elif award == 'result':
                    n += v

                elif award_is_callable:
                    n += award(v)

                else:
                    n += award

        if commit:
            self._commit('_networth', n)

        return n

    def _commit(self, field, value):
        raise NotImplementedError


__author__ = 'heddevanderheide'

# Django specific
from django.db import models
from django.db.models import Max
from django.db.models.query import QuerySet


class NetworthQuerySet(QuerySet):
    def ceiling(self):
        return self.aggregate(Max('_networth'))['_networth__max']


class NetworthManager(models.Manager):
    def get_queryset(self):
        return NetworthQuerySet(self.model, using=self._db)